package com.tongda.tdpimssdk.remote.entity;

import com.alibaba.fastjson.JSONObject;
import com.tongda.tdpimssdk.http.PimHttpUtil;
import com.tongda.tdpimssdk.remote.RemoteSaAuthHandler;
import com.tongda.tdpimssdk.remote.common.RemoteAbstractEntity;
import com.tongda.tdpimssdk.remote.paramset.AssetParamSet;
import com.tongda.tdpimssdk.remote.paramset.TeamParamSet;
import lombok.Data;

import javax.xml.bind.annotation.XmlAttribute;
import java.io.FileNotFoundException;

import static com.tongda.tdpimssdk.http.PimHttpUtil.resultUtil;

@Data
public class RemoteDocument extends RemoteAbstractEntity {

    public static final String METADATA_URI = "/pims/asset/document/field";
    public static final String BASE_URI = "/pims/asset/document";
    public static final String UPLOAD_URI = "/pims/document/upload";
    public static final String BASE_URI_UPGRADE = "/pims/document/upgrade";
    public static final String BASE_URI_LIFECYCLE = "/pims/lifecycle/document/state";


    @XmlAttribute(name = "href")
    protected String href;
    @XmlAttribute(name = "mediatype")
    protected String mediaType;


    public AssetParamSet prepareMetadataUpdate(){
        this.assetVo= new AssetParamSet(this.getRsh(), AssetParamSet.action_document_metadata_update);
        this.assetVo.setId(this.getId());
        return this.assetVo;
    }

    public AssetParamSet prepareUploadBinary() throws FileNotFoundException {
            this.assetVo= new AssetParamSet(this.getRsh(), AssetParamSet.action_document_file_upload);
            this.assetVo.setId(this.getId());
            return this.assetVo;
    }

    public AssetParamSet prepareChangeLifeCycleStatus() {
        this.assetVo= new AssetParamSet(this.getRsh(), AssetParamSet.action_document_change_lifecycle);
        this.assetVo.setId(this.getId());
        return this.assetVo;
    }

    public AssetParamSet prepareUpgradeVersion(){
        this.assetVo= new AssetParamSet(this.getRsh(), AssetParamSet.action_document_upgrade);
        this.assetVo.setId(this.getId());
        return this.assetVo;
    }



    public static RemoteDocument doFetchDocument(RemoteSaAuthHandler rsa, String docID){
        String url = "http://" + rsa.serverIP + ":" + rsa.serverPort + "/pims/document/content?dbName=" + rsa.dbName + "&id=" + docID;
        String response = PimHttpUtil.sendGet(rsa.saToken, url);
        String s = resultUtil(response);
        RemoteDocument remoteDocument = JSONObject.parseArray(s, RemoteDocument.class).get(0);
        remoteDocument.setRsh(rsa);
        return remoteDocument;
    }

    public static String doDeleteDocument(RemoteSaAuthHandler rsa, String docID) {
        AssetParamSet param = new AssetParamSet(rsa, AssetParamSet.action_document_delete);
        param.setId(docID);
        return param.doAction();
    }

    public static AssetParamSet prepareToCreateDocument(RemoteSaAuthHandler rsa, String templateID, String label, String creatorID) {
        AssetParamSet param = new AssetParamSet(rsa, AssetParamSet.action_document_create);
        param.setParamTemplate(templateID)
                .setParamLabel(label)
                .setParamUserID(creatorID);
        return param;
    }

}
