package com.tongda.tdpimssdk.remote.paramset;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.tongda.tdpimssdk.http.PimHttpUtil;
import com.tongda.tdpimssdk.remote.entity.RemoteAsset;
import com.tongda.tdpimssdk.remote.entity.RemoteDocument;
import com.tongda.tdpimssdk.remote.RemoteSaAuthHandler;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.tongda.tdpimssdk.http.PimHttpUtil.resultUtil;

/**
 * @Tittle: AssetVo
 * @description:
 * @Package: com.tongda.pimdemo.http.resultVo.vo
 * @Date 2023/5/25 19:10
 */
@Data
public class AssetParamSet {

    public static final int action_asset_create =100;
    public static final int action_asset_metadata_update = 101;
    public static final int action_asset_delete = 102;
    public static final int action_asset_upgrade = 103;
    public static final int action_asset_change_lifecycle = 104;




    public static final int action_document_metadata_update = 200;
    public static final int action_document_create = 201;
    public static final int action_document_delete = 202;
    public static final int action_document_file_upload = 203;
    public static final int action_document_upgrade = 204;
    public static final int action_document_change_lifecycle = 205;



    @JSONField(serialize = false)
    public int actionType = 0;

    @JSONField(serialize = false)
    protected RemoteSaAuthHandler rsh;

    //用于创建资产的基本属性信息
    private String templateID;

    private String label;
    private String userID;
    private String userName;


    //用于修改元数据、上传流文件、删除资产的实体对象ID, 升级版本，变更生命周期状态
    private String id;
    private String desc;
    private String lcEvent;



    //用于文档上传二进制流文件所需的属性信息
    private String mediaType;
    private String upLoadFilePath;



    //用于创建和修改元数据字段的属性信息
    private List<AssetField> fields;


    public AssetParamSet(){
    }
    public AssetParamSet(RemoteSaAuthHandler rsh, int actionType){
            this.rsh = rsh;
            this.actionType = actionType;
    }


    public AssetParamSet setAssetField(AssetField field){
        if(fields == null){
            this.fields = new ArrayList<>();
        }
        this.fields.add(field);
        return this;
    }



    /**
     * '[
     * {"key": "tdpim.asset.mainframe.cpu.count", "type":"SingleValue", "value":"6"},
     * {"key": "tdpim.asset.mainframe.cpu.temp", "type":"Range", "valueMin":"1", "valueMax":"5"},
     * {"key": "tdpim.asset.mainframe.area", "type":"MultiSelect", "selectedIDs":["CandidateValue-1110951999471353858","CandidateValue-1110951999471353859"]}
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    public static class AssetField {

        //元数据Key
        private String key;

        //元数据模式
        private String mode;  //元数据模式

        //元数据据计量单位
        private String uom;


//单值
        private String value;

// 范围

        private String valueMin;
        private String valueMax;


// 多选
        private List<String> selectedIDs;


        public AssetField(String key, String value){
            this.setKey(key);
            this.setValue(value);
        }

        public AssetField(String key, Integer value, String uom){
            this.setKey(key);
            this.setIntegerValue(value);
            this.setUom(uom);
        }

        public AssetField(String key, Float value, String uom){
            this.setKey(key);
            this.setFloatValue(value);
            this.setUom(uom);
        }

        public AssetField(String key, Integer valueMin, Integer valueMax, String uom){
            this.setKey(key);
            this.setIntegerValue(valueMin);
            this.setIntegerValue(valueMax);
            this.setUom(uom);
        }

        public AssetField(String key, Integer[] values, String uom){
            this.setKey(key);
            this.setMultiValue(values);
            this.setUom(uom);
        }

        public AssetField(String key, Float[] values, String uom){
            this.setKey(key);
            this.setMultiValue(values);
            this.setUom(uom);
        }

        public AssetField(String key, String[] values){
            this.setKey(key);
            this.setMultiValue(values);
        }


        /**
         * 添加完整的构造方法
         */


        public void setIntegerValue(Integer value){
            this.setValue(String.valueOf(value));
        }

        public void setFloatValue(Float value){
            this.setValue(String.valueOf(value));
        }

        public void setMinValue(Integer value){
            this.setValueMin(String.valueOf(value));
        }

        public void setMinValue(Float value){
            this.setValueMin(String.valueOf(value));
        }

        public void setMaxValue(Integer value){
            this.setValueMax(String.valueOf(value));
        }

        public void setMaxValue(Float value){
            this.setValueMax(String.valueOf(value));
        }

        public void setMultiValue(Integer[] value){
            String v = "";
            for(Integer i : value){
                v += String.valueOf(i) + ";";
            }
            this.setValue(v);
        }

        public void setMultiValue(Float[] value){
            String v = "";
            for(Float i : value){
                v += String.valueOf(i) + ";";
            }
            this.setValue(v);
        }

        public void setMultiValue(String[] value){
            String v = "";
            for(String i : value){
                v += i + ";";
            }
            this.setValue(v);
        }

    }


    public AssetParamSet setParamTemplate(String templateID){
        this.setTemplateID(templateID);
        return this;
    }

    public AssetParamSet setParamLabel(String label){
        this.setLabel(label);
        return this;
    }

    public AssetParamSet setParamUserID(String userID){
        this.setUserID(label);
        return this;
    }

    public AssetParamSet setParamUploadFilePath(String path){
        this.setUpLoadFilePath(path);
        return this;
    }

    public AssetParamSet setParamMediaType(String mediaType){
        this.setMediaType(mediaType);
        return this;
    }

    public AssetParamSet setParamDesc(String str){
        this.setDesc(str);
        return this;
    }

    public AssetParamSet setParamLifeCycleEvent(String str){
        this.setLcEvent(str);
        return this;
    }




    //资产创建和元数据修改
    private String doCreateAsset(){
        String url="http://"+ this.rsh.serverIP +":"+ this.rsh.serverPort + RemoteAsset.BASE_URI + "?dbName="+ this.rsh.dbName;
        String response= PimHttpUtil.sendPost(this.rsh.saToken,url,(JSONObject)JSONObject.toJSON(this));
        String s= resultUtil(response);
        return s;
    }

    private String doUpdateAssetMetadataValues(){
        String url="http://"+ this.rsh.serverIP +":"+ rsh.serverPort+ RemoteAsset.METADATA_URI+"?dbName="+ this.rsh.dbName;
        String response= PimHttpUtil.sendPut(this.rsh.saToken,url,(JSONObject)JSONObject.toJSON(this));
        String s=  resultUtil(response);
        return s;
    }

    //文档创建和元数据修改
    private String doCreateDocument(){
        String url="http://"+ this.rsh.serverIP +":"+ this.rsh.serverPort+ RemoteDocument.BASE_URI+"?dbName="+ this.rsh.dbName;
        String response= PimHttpUtil.sendPost(this.rsh.saToken,url,(JSONObject)JSONObject.toJSON(this));
        String s=  resultUtil(response);
        return s;
    }

    private String doUpdateDocumentMetadataValues(){
        String url="http://"+ this.rsh.serverIP +":"+ this.rsh.serverPort+ RemoteDocument.METADATA_URI+"?dbName="+ this.rsh.dbName;
        String response= PimHttpUtil.sendPut(this.rsh.saToken,url,(JSONObject)JSONObject.toJSON(this));
        String s=  resultUtil(response);
        return s;
    }

    private String doDeleteDocument(){
        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + RemoteDocument.BASE_URI+ "?dbName=" + this.rsh.dbName + "&id=" + this.getId();
        String response = PimHttpUtil.sendDelete(this.rsh.saToken, url);
        String s = resultUtil(response);
        return s;
    }

    private String doDeleteAsset(){
        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + RemoteAsset.BASE_URI  +"dbName=" + this.rsh.dbName + "&id=" +  this.getId();
        String response = PimHttpUtil.sendDelete(this.rsh.saToken, url);
        String s = resultUtil(response);
        return s;
    }

    private String doUploadFile() {

        File file  = new File(this.getUpLoadFilePath());
        if(!file.exists()){
           return "Error: 文件未找到:" + this.getUpLoadFilePath();
        }
        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + RemoteDocument.UPLOAD_URI + "?dbName=" + this.rsh.dbName + "&id=" + this.getId();
        String response = PimHttpUtil.uploadFile(this.rsh.saToken, url, file);
        String s = resultUtil(response);
        return s;
    }

    private String doAssetUpgradeVersion(){
        //{docID: "AST_1138852517807915008", desc: "1111"}

        Map<String,String> param = new HashMap<>();
        param.put("docID", this.getId());
        param.put("desc", this.getDesc());

        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + RemoteAsset.BASE_URI_UPGRADE  +"?dbName=" + this.rsh.dbName + "&id=" +  this.getId();
        String response = PimHttpUtil.sendPost(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(param));
        String s = resultUtil(response);
        return s;
    }

    private String doDocumentUpgradeVersion(){

        Map<String,String> param = new HashMap<>();
        param.put("docID", this.getId());
        param.put("desc", this.getDesc());

        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + RemoteDocument.BASE_URI_UPGRADE  +"?dbName=" + this.rsh.dbName + "&id=" +  this.getId();
        String response = PimHttpUtil.sendPost(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(param));
        String s = resultUtil(response);
        return s;
    }

    private String doAssetNewLifeCycleStatus(){
        Map<String,Object> param = new HashMap<>();
        param.put("entityId", this.getId());

        Map<String,String> event = new HashMap<>();
        event.put("key", this.getLcEvent());
        //"Submit"
        param.put("tdEvent", event);

        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + RemoteAsset.BASE_URI_LIFECYCLE  +"?dbName=" + this.rsh.dbName ;
        String response = PimHttpUtil.sendPut(this.rsh.saToken, url, (JSONObject)JSONObject.toJSON(param));
        String s = resultUtil(response);
        return s;
    }

    private String doDocumentNewLifeCycleStatus(){

        Map<String,Object> param = new HashMap<>();
        param.put("entityId", this.getId());

        Map<String,String> event = new HashMap<>();
        event.put("key", this.getLcEvent());
        //"Submit"
        param.put("tdEvent", event);

        System.out.println(JSONObject.toJSON(param));

        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + RemoteDocument.BASE_URI_LIFECYCLE  +"?dbName=" + this.rsh.dbName ;
        String response = PimHttpUtil.sendPut(this.rsh.saToken, url, (JSONObject)JSONObject.toJSON(param));
        String s = resultUtil(response);
        return s;
    }

    public String doAction(){

        System.out.println(JSON.toJSONString(this));

        switch (this.actionType) {
            case AssetParamSet.action_asset_create :
                return doCreateAsset();
            case AssetParamSet.action_document_create :
                return doCreateDocument();
            case AssetParamSet.action_document_metadata_update :
                return doUpdateDocumentMetadataValues();
            case AssetParamSet.action_asset_metadata_update:
                return doUpdateAssetMetadataValues();
            case AssetParamSet.action_document_delete:
                return doDeleteDocument();
            case AssetParamSet.action_asset_delete:
                return doDeleteAsset();
            case AssetParamSet.action_document_file_upload:
                    return doUploadFile();
            case AssetParamSet.action_asset_upgrade:
                return doAssetUpgradeVersion();
            case AssetParamSet.action_asset_change_lifecycle:
                return doAssetNewLifeCycleStatus();
            case AssetParamSet.action_document_upgrade:
                return doDocumentUpgradeVersion();
            case AssetParamSet.action_document_change_lifecycle:
                return doDocumentNewLifeCycleStatus();
            default:
                return null;
        }

    }
}
