package com.tongda.tdpimssdk.remote.paramset;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.tongda.tdpimssdk.http.PimHttpUtil;
import com.tongda.tdpimssdk.remote.RemoteSaAuthHandler;
import com.tongda.tdpimssdk.remote.entity.RemoteTeam;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;
import java.util.List;

import static com.tongda.tdpimssdk.http.PimHttpUtil.resultUtil;

@Data
public class TeamParamSet {

    @JSONField(serialize = false)
    protected RemoteSaAuthHandler rsh;

    public static final int action_team_create =100;

    public static final int action_team_upgrade_version =101;

    public static final int action_team_change_lifecycle = 102;

    public static final int action_team_delete = 103;

    public static final int action_team_add_asset_ref = 104;

    public static final int action_team_remove_asset_ref = 105;

    @JSONField(serialize = false)
    public int actionType;

    //用户创建业务
    private String userId;
    private String label;
    private String desc;
    private String lifecycleID;



    //修改业务内部数据，变更状态，提升版本
    private String teamID;

    private String lcEvent;


    //添加资源
    private List<String> assetListToAdd;


    private String assetToRemove;


    public TeamParamSet(RemoteSaAuthHandler rsh, int actionType){
        this.rsh = rsh;
        this.actionType = actionType;
    }


    public TeamParamSet setParamUserID(String str){
        this.setUserId(str);
        return this;
    }

    public TeamParamSet setParamLabel(String str){
        this.setLabel(str);
        return this;
    }

    public TeamParamSet setParamDesc(String str){
        this.setDesc(str);
        return this;
    }

    public TeamParamSet setParamLifecycleID(String str){
        this.setLifecycleID(str);
        return this;
    }

    public TeamParamSet setParamLcEvent(String str){
        this.setLcEvent(str);
        return this;
    }

    public TeamParamSet setParamAssetListToAdd(List<String> assetIDList){
        this.setAssetListToAdd(assetIDList);
        return this;
    }

    public TeamParamSet setParamAssetToRemove(String assetID){
        this.setAssetToRemove(assetID);
        return this;
    }



    private String doCreateTeam(){
        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort +  RemoteTeam.BASE_URI  + "?dbName=" + this.rsh.dbName;
        String response = PimHttpUtil.sendPost(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(this));
        String s = resultUtil(response);
        return s;
    }

    private String doDeleteTeam(){
        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort +  RemoteTeam.BASE_URI  + "?dbName=" + this.rsh.dbName;
        String response = PimHttpUtil.sendDeleteBody(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(this));
        String s = resultUtil(response);
        return s;
    }

    private String doUpgradeVersion(){
        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort +  RemoteTeam.BASE_URI_UPGRADE_VERSION  + "?dbName=" + this.rsh.dbName;
        String response = PimHttpUtil.sendPost(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(this));
        String s = resultUtil(response);
        return s;
    }

    private String doAssetNewLifeCycleStatus(){

        Map<String,Object> param = new HashMap<>();
        param.put("entityId", this.getTeamID());

        Map<String,String> event = new HashMap<>();
        event.put("key", this.getLcEvent());
        //"Submit"
        param.put("tdEvent", event);


        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort +  RemoteTeam.BASE_URI_LIFECYCLE  + "?dbName=" + this.rsh.dbName;
        String response = PimHttpUtil.sendPut(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(param));
        String s = resultUtil(response);
        return s;
    }


    private String doAddAssetRef(){

        Map<String,Object> param = new HashMap<>();
        param.put("id", this.getTeamID());

        String ids = "";
        for(String af : this.assetListToAdd){
            ids += af + ",";
        }
        param.put("assetID", ids);

        System.out.println(JSONObject.toJSON(param));

        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort +  RemoteTeam.BASE_URI_ASSET + "?dbName=" + this.rsh.dbName;
        System.out.println(url);
        String response = PimHttpUtil.sendPut(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(param));
        String s = resultUtil(response);
        return s;
    }

    private String doRemoveAssetRef(){

        Map<String,Object> param = new HashMap<>();

        param.put("id", this.getTeamID());
        param.put("assetID", this.assetToRemove);

        System.out.println(JSONObject.toJSON(param));

        String url = "http://" + this.rsh.serverIP + ":" + this.rsh.serverPort + "/pims/business/asset?dbName=" + this.rsh.dbName;
        String response = PimHttpUtil.sendDeleteBody(this.rsh.saToken, url, (JSONObject) JSONObject.toJSON(param));
        String s = resultUtil(response);

        return s;
    }

    public String doAction() {

        switch (this.actionType) {
            case TeamParamSet.action_team_create:
                return doCreateTeam();
            case TeamParamSet.action_team_upgrade_version:
                return doUpgradeVersion();
            case TeamParamSet.action_team_change_lifecycle:
                return doAssetNewLifeCycleStatus();
            case TeamParamSet.action_team_delete:
                return doDeleteTeam();
            case TeamParamSet.action_team_add_asset_ref:
                return doAddAssetRef();
            case TeamParamSet.action_team_remove_asset_ref:
                return doRemoveAssetRef();
            default:
                return null;
        }
    }


}
